"""Library for temporarily storing functions as modules.

Python library for depositing a function definition inside
a temporary module file.
"""

import doctest
import importlib
import os
import types
import inspect

def capsules(function):
    """
    Save supplied function definition to a module file,
    load that module file, and returned the function from
    the loaded module.
    """
    if not isinstance(function, types.FunctionType):
        raise TypeError('input is not a function')

    if function.__name__ == '<lambda>':
        raise ValueError('lambda functions are not supported')

    try:
        source = inspect.getsource(function)
    except:
        raise

    context_qualified_module = __name__
    context_qualified_path = __name__.replace(".", "/")
    directory = '/__capsules__/' + context_qualified_path
    module_name = '__capsules__.' + context_qualified_module + '.' + function.__name__

    if not os.path.exists('.' + directory):
        os.makedirs('.' + directory)

    path_file = '.' + directory + '/' + function.__name__ + '.py'
    with open(path_file, 'w') as file:
        definition = source
        lines = definition.split('\n')
        if lines[0] == '@capsules':
            definition = '\n'.join(lines[1:])
        file.write(definition)

    spec = importlib.util.spec_from_file_location(module_name, path_file)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return getattr(module, function.__name__)

if __name__ == "__main__":
    doctest.testmod()  # pragma: no cover



import os
import shutil
from unittest import TestCase

from capsules.capsules import capsules

# Remove directory if it exists to ensure
# tests have full coverage.
if os.path.exists('./__capsules__'):
    shutil.rmtree('./__capsules__')

# Definitions of functions used in unit tests.
global identity
identity = lambda x: x

global double
@capsules
def double(x):
    return x + x

global square
def square(x):
    return x * x

global power
exec('def power(x): return x**x')

class Test_capsules(TestCase):
    def test_non_function(self):
        self.assertRaises(TypeError, capsules, 123)
        try:
            capsules(123)
            self.assertFail()
        except TypeError as error:
            self.assertEqual(str(error), 'input is not a function')

    def test_lambda(self):
        global identity
        self.assertRaises(ValueError, capsules, identity)
        try:
            capsules(identity)
            self.assertFail()
        except ValueError as error:
            self.assertEqual(str(error), 'lambda functions are not supported')

    def test_no_source(self):
        global power
        self.assertRaises(OSError, capsules, power)
        try:
            capsules(power)
            self.assertFail()
        except OSError as error:
            self.assertEqual(str(error), 'could not get source code')

    def test_decorator(self):
        global double
        self.assertTrue(double(2) == 4)

    def test_direct(self):
        global square
        square = capsules(square)
        self.assertTrue(square(3) == 9)
